// ----------------------------------------------------------------------------
// visual.cpp
// ----------------------------------------------------------------------------
//
// Author: Daniel Mendoza
//
// 
// ----------------------------------------------------------------------------



#include <iostream>
#include <map>

#include "risc.h"

void print_space() {
	std::cout << "     ";
}

void print_down_dash() {
	std::cout << "|    ";
}

void print_name(std::string name) {
	std::cout << "----" << name;
}

void print_line_levels(int level, std::map<int,bool> &open_levels) {
	if(level != 0) print_space();
	for(int i = 0; i < level; i++) {
			if(open_levels[i] && i < level - 1){
				print_down_dash();
			}
			else if (i == level - 1) {
				std::cout << "|";
			}
			else {
				print_space();
			}
	}
}

void print_ports_helper(int level,std::map<int,bool> &open_levels,risc::Port* port,bool helper_delete_condition,risc::InstanceTree &instance_tracker) {
	
	risc::InstanceTree mapped_channels = risc::get_mapped_channel_instance(instance_tracker);
	
	print_line_levels(level,open_levels);
	std::cout << std::endl;
	print_line_levels(level,open_levels);		
	std::cout << "-" << port->get_ast_type_name() << " " << port->get_name() << "->";
	std::cout << std::endl;			
	if(helper_delete_condition) {
		open_levels.erase(level - 1);
	}

	level++;

		print_line_levels(level,open_levels);
		std::cout << "->";
		for(std::list<risc::InstanceTree::Element>::iterator it = mapped_channels.path_.begin(); it != mapped_channels.path_.end();it++) {
			if(it->second) {
				std::cout << it->second->get_ast_type_name() << " " << it->second->get_name();
			}
			else {
				std::cout << "Not defined";
			}
			if(&(*it) != &mapped_channels.path_.back()) {
				std::cout << "->";
			}
			else {
				std::cout << std::endl;
			} 
		}
		

}

/**
 * \fn		print_ports
 * \brief	This function analyzes the ports of a modules and outputs the port
 * 		hierarchy
 */
void print_ports(risc::Instance* m, int level, std::map<int,bool> &open_levels, bool delete_condition, risc::InstanceTree &instance_tracker) {
	bool local_delete_condition = false;
	for(risc::InPortVectorIter inport_iter = m->get_definition()->inports_.begin();
		inport_iter != m->get_definition()->inports_.end();
		inport_iter++) {
	
			if((*inport_iter) == m->get_definition()->inports_.back() && m->get_definition()->inoutports_.empty() && m->get_definition()->outports_.empty()) {
				local_delete_condition = true;
			}

			risc::InstanceTree::Element port_instance(m->get_definition(),(*inport_iter));
			instance_tracker.path_.push_back(port_instance);
	
			print_ports_helper(level,open_levels,(*inport_iter),local_delete_condition && delete_condition,instance_tracker);
	
			instance_tracker.path_.pop_back();
	}
	
	for(risc::InOutPortVectorIter inoutport_iter = m->get_definition()->inoutports_.begin();	
		inoutport_iter != m->get_definition()->inoutports_.end();
		inoutport_iter++) {

			if((*inoutport_iter) == m->get_definition()->inoutports_.back() && m->get_definition()->outports_.empty()) {
				local_delete_condition = true;
			}
			risc::InstanceTree::Element port_instance(m->get_definition(),(*inoutport_iter));
			instance_tracker.path_.push_back(port_instance);
	
			print_ports_helper(level,open_levels,(*inoutport_iter),local_delete_condition && delete_condition,instance_tracker);
	
			instance_tracker.path_.pop_back();
	}
	
	for(risc::OutPortVectorIter outport_iter = m->get_definition()->outports_.begin();
		outport_iter != m->get_definition()->outports_.end();
		outport_iter++) {

			if((*outport_iter) == m->get_definition()->outports_.back()) {
				local_delete_condition = true;
			}
			risc::InstanceTree::Element port_instance(m->get_definition(),(*outport_iter));
			instance_tracker.path_.push_back(port_instance);
	
			print_ports_helper(level,open_levels,(*outport_iter),local_delete_condition && delete_condition,instance_tracker);
	
			instance_tracker.path_.pop_back();
	}
}

/**
 * \fn		print_wrapper
 * \brief	This function is the main control flow for outputting the module
 * 		hierarchy
 */
void print_wrapper (risc::Instance* m,risc::InstanceTree &instance_tracker) {
	static bool delete_condition = false;
	static int level = -1;
	static std::map<int,bool> open_levels;
	

	level++;

	print_line_levels(level, open_levels);	
	std::cout << std::endl;
	print_line_levels(level, open_levels);
	print_name(m->get_ast_type_name() + " " + m->get_name());
	std::cout << std::endl;

	if(delete_condition) {
		delete_condition = false;
		open_levels.erase(level - 1);
	}

	if(!m->get_definition()->sub_modules_.empty()
		|| !m->get_definition()->sub_hier_channel_.empty()
		|| !m->get_definition()->variables_.empty()
		|| !m->get_definition()->inoutports_.empty()
		|| !m->get_definition()->outports_.empty()
		|| !m->get_definition()->inports_.empty()) {
			open_levels[level] = true;
	}
	else {
		level--;
		return;
	}

	if(!m->get_definition()->inoutports_.empty()
		|| !m->get_definition()->outports_.empty()
		|| !m->get_definition()->inports_.empty()) {
		level++;
		print_ports(m,level,open_levels,m->get_definition()->variables_.empty() && m->get_definition()->sub_modules_.empty() && m->get_definition()->sub_hier_channel_.empty(),instance_tracker);
		level--;
	}

	if(!m->get_definition()->variables_.empty()) {
		level++;
		for(risc::VariableVectorIter iter = m->get_definition()->variables_.begin();
		iter != m->get_definition()->variables_.end();
		iter++) {
			print_line_levels(level,open_levels);
			std::cout << std::endl;
			print_line_levels(level,open_levels);
			print_name((*iter)->get_ast_type_name() + " " + (*iter)->get_name());
			std::cout << std::endl;
		}
		level--;
	}

	
	for(risc::ModuleInstanceVectorIter iter = m->get_definition()->sub_modules_.begin();
        iter != m->get_definition()->sub_modules_.end();
        iter++) {
		if((*iter) == m->get_definition()->sub_modules_.back() && m->get_definition()->sub_hier_channel_.empty()) {
			delete_condition = true;
		}
		risc::InstanceTree::Element e(m->get_definition(),(*iter));
		instance_tracker.path_.push_back(e);		
                print_wrapper((*iter),instance_tracker);
		instance_tracker.path_.pop_back();
        }
        for(risc::HierarchicalChannelInstanceVectorIter iter = m->get_definition()->sub_hier_channel_.begin();
        iter != m->get_definition()->sub_hier_channel_.end();
        iter++) {
		if((*iter) == m->get_definition()->sub_hier_channel_.back()) {
			delete_condition = true;
		}
                print_wrapper((*iter),instance_tracker);
        }

	level--;

}

int main(int argc, char* argv[]) {

	for(int i = 0; i < argc; i++) {
		std::string str(argv[i]);
		if(str == "-h" || str == "--help") {
			std::cout << "To use tree tool please use the command line format: ./tree [SystemC File] [SystemC Include/Library Flags]" << std::endl;
		return 0;
		}
	}

	std::string LWIncludePath;
	LWIncludePath = "-I";
	LWIncludePath += getenv(SYSTEMC_LW_HOME_ENV);
	LWIncludePath += "/include";
	
	// add -I$SYSTEMC_LW_HOME/include
	risc::tools::AddCommandlineOption(argc, argv, LWIncludePath);


	std::cout << std::endl
            	<< "start processing file: "
            	<< " ..."
            	<< std::endl;

	risc::Design d(argc,argv);

	risc::InstanceTree instance_tracker = d.get_root();

	std::cout << "+++++++++++++++++++ Hierarchy Modules ++++++++++++++++++++"
            	<< std::endl;
	
	for(risc::InstanceVectorIter iter = d.top_modules_.begin();
		iter != d.top_modules_.end();
		iter++) {
			print_wrapper((*iter),instance_tracker);
	}
	
	std::cout << std::endl
		<<"end processing file: "
		<< " ..."
		<<std::endl;

return 0;
}
