#ifndef HIERARCHICAL_CHANNEL_INSTANCE_H_INCLUDED_
#define HIERARCHICAL_CHANNEL_INSTANCE_H_INCLUDED_

#include <vector>

#include "rose.h"

#include "class.h" // included for get_definition
#include "hierarchical_channel.h" // included for get_definition

#include "module_instance.h"

namespace risc {

class HierarchicalChannelInstance: public ModuleInstance {

public:

  /**
   * \brief default constructor
   * \param ast_node  associated definition node in the ast
   * \param ir_def    internal definition pointer of the instance
   */
  explicit HierarchicalChannelInstance(
      SgVariableDefinition *ast_node,
      HierarchicalChannel*ir_def);


  /**
   * \fn       get_definition
   * \brief    returns this associated definition of the hierarchical channel
   *           instance
   */
  virtual HierarchicalChannel* get_definition();

private:

  /**
   * \brief This pointer points to the corresponding definition in the
   *        internal representation
   */
  HierarchicalChannel *channel_definition_;

  /**
   * \brief Disabled copy constructor
   */
  HierarchicalChannelInstance(const HierarchicalChannelInstance &hci);
};

typedef std::vector<HierarchicalChannelInstance*>
            HierarchicalChannelInstanceVector;
typedef std::vector<HierarchicalChannelInstance*>::iterator
            HierarchicalChannelInstanceVectorIter;
typedef std::vector<HierarchicalChannelInstance*>::const_iterator
            HierarchicalChannelInstanceVectorConstIter;

}; // end namespace risc

#endif /* HIERARCHICAL_CHANNEL_INSTANCE_H_INCLUDED_ */

/* ex: set softtabstop=2 tabstop=2 shiftwidth=2 expandtab: */
