#ifndef PORT_MAPPING_CACHE_H_INCLUDED_
#define PORT_MAPPING_CACHE_H_INCLUDED_

#include <unordered_map>

#include "rose.h"

#include "instance_tree.h"
#include "path_instance_mapper.h"

namespace risc {

/**
 * This class caches the mapping of ports and the mapped channels
 */
class PortMappingCache {

public:
  PortMappingCache();

  /**
   * \brief This maps stores the mapping for ports to channels
   *        (instance id of the module where the port is declared)
   *         ->
   *        variable definiton of the port
   *        (path to the mapped channel, instance id of the channel)
   */
  std::unordered_map<int,
                     std::unordered_map<SgVariableDefinition*,
                                        std::pair<risc::InstanceTree,
                                                  int> > > mapping_;

  /**
   * \fn    initialize
   * \brief This function initializes the mapping.
   */
  void initialize(PathInstanceMapper *path_instance_mapper);

  void print();

  /**
   * \brief This returns for a given channel instance the mapped modules
   */
  std::list<std::pair<int, InstanceTree>>
  get_mapped_module(int channel_id, SgClassDefinition *channel_definition);

private:
  bool is_initialized_;
};

}; // end of namespace risc

#endif /* PORT_MAPPING_CACHE_H_INCLUDED_ */
