#include "time.h"

#include<cassert>

risc::sg::Time::Time():
  time_units_(0),
  delta_count_(0),
  infinite_(false)
{ }

risc::sg::Time::Time(unsigned long long time_units, int delta_count):
  time_units_(time_units),
  delta_count_(delta_count),
  infinite_(false)
{ }

risc::sg::Time::Time(unsigned long long time_units, int delta_count, bool infinite):
  time_units_(time_units),
  delta_count_(delta_count),
  infinite_(infinite)
{ }

risc::sg::Time::Time(bool infinite):
  time_units_(0),
  delta_count_(0),
  infinite_(infinite)
{ }

bool
risc::sg::Time::operator<(const Time &ohs)
{
  if(infinite_) {
    return false;
  }

  if(ohs.infinite_) {
    return true;
  }

  if(time_units_ < ohs.time_units_) {
    return true;
  }

  if(time_units_ == ohs.time_units_ &&
      delta_count_ < ohs.delta_count_) {
    return true;
  }

  return false;
}

bool
risc::sg::Time::operator>(const Time &ohs)
{
  if(infinite_) {
    return true;
  }

  if(ohs.infinite_) {
    return false;
  }

  if(time_units_ > ohs.time_units_) {
    return true;
  }

  if(time_units_ == ohs.time_units_ &&
      delta_count_ > ohs.delta_count_) {
    return true;
  }

  return false;
}

bool
risc::sg::Time::operator<=(const Time &ohs)
{
  return (*this < ohs) || (*this == ohs);
}

bool
risc::sg::Time::operator>=(const Time &ohs)
{
  return (*this > ohs) || (*this == ohs);
}

bool
risc::sg::Time::operator==(const Time &ohs)
{
  return time_units_ == ohs.time_units_ &&
    delta_count_ == ohs.delta_count_;
}

bool
risc::sg::Time::operator!=(const Time &ohs)
{
  return !(*this == ohs);
}

risc::sg::Time
risc::sg::Time::operator+(const Time &ohs)
{
  if(ohs.time_units_ == 0) {
    return Time(time_units_, delta_count_ + ohs.delta_count_);
  }

  return Time(ohs.time_units_ + time_units_, ohs.delta_count_);
}

risc::sg::Time&
risc::sg::Time::operator+=(const Time &ohs)
{
  *this = *this + ohs;
  return *this;
}

bool
risc::sg::Time::one_delta_cycle_ahead(const Time &ohs)
{
  return (time_units_ == ohs.time_units_)
    && (delta_count_ == (ohs.delta_count_ - 1));
}

std::ostream&
risc::sg::operator<<(std::ostream& os, const risc::sg::Time& time)
{
  if(time.infinite_) {

    os << "(&#8734;:&#8734;)";
  } else {
    os << "(" << time.time_units_ << ":" << time.delta_count_ << ")";
  }
  return os;
}

long long risc::sg::Time::convert_si_magnitude(std::string magnitude)
{
  if("SC_FS" == magnitude) {
    return 1;

  } else if("SC_PS" == magnitude) {
    return 1000;

  } else if("SC_NS" == magnitude) {
    return 1000000;

  } else if("SC_US" == magnitude) {
    return 1000000000;

  } else if("SC_MS" == magnitude) {
    return 1000000000000;

  } else if("SC_SEC" == magnitude) {
    return 1000000000000000;

  } else {
    assert("Unsupported magnitude" && 0);
  }
}

/* ex: set softtabstop=2 tabstop=2 shiftwidth=2 expandtab: */
