// green_example.cc
// 04/11/17 RD   added assertions
// 04/04/17 ZC   initial version

#include <stdio.h>
#include "systemc.h"

#define ITERATIONS 500

class Main_Block : public sc_module
{
    public:
	sc_event e;
	int s, r;

	void S()
	{
		printf("S starting\n");
		s = 1;
		wait(1,SC_NS);
		e.notify(SC_ZERO_TIME);
		s = 2;
		printf("S done\n");
	}

	void R()
	{
		printf("R starting\n");
		r = 1;
		wait(e);
		printf("R done\n");
		r = 2;
	}

	Main_Block(sc_module_name name)
	: sc_module(name)
	{
		s = r = 0;
		SC_THREAD(S);
		SC_THREAD(R);
	}
	SC_HAS_PROCESS(Main_Block);
};

class W : public sc_module
{
    public:
	int w1, w2;

	void W1()
	{
		printf("W1 starting\n");
		w1 = 1;
		for(int i=0;i<ITERATIONS;i++){
			wait(1,SC_NS);
			w1++;
		}
		assert(w1 == ITERATIONS+1);
		printf("W1 done\n");
	}

	void W2()
	{
		printf("W2 starting\n");
		w2 = 1;
		for(int i=0;i<ITERATIONS;i++){
			wait(2,SC_NS);
			w2++;
		}
		assert(w2 == ITERATIONS+1);
		printf("W2 done\n");
	}

	W(sc_module_name name)
	: sc_module(name)
	{
		w1 = w2 = 0;
		SC_THREAD(W1);
		SC_THREAD(W2);
	}
	SC_HAS_PROCESS(W);
};

class Platform : public sc_module
{
    public:
	W w;
	Main_Block m;

	Platform(sc_module_name name)
	: sc_module(name),
	w("w"),
	m("m")
	{
		printf("green example constructor\n");
	}
	SC_HAS_PROCESS(Platform);
};

int sc_main(int argc, char* argv[])
{
    Platform p("p");

    printf("one_notify_one_wait starting\n");
    assert(p.m.s == 0);
    assert(p.m.r == 0);
    assert(p.w.w1 == 0);
    assert(p.w.w2 == 0);
    sc_start();
    assert(p.m.s == 2);
    assert(p.m.r == 2);
    assert(p.w.w1 == ITERATIONS+1);
    assert(p.w.w2 == ITERATIONS+1);
    printf("one_notify_one_wait done\n");
    return 0;
}

/* ex: set softtabstop=8 tabstop=8 shiftwidth=8 expandtab: */
