// prod_cons_signal.cc
//
// 06/12/18 RD	combined/extended version
// 06/11/18 DM	original versions (single, multiple signals)

#include <assert.h>
#include "systemc.h"

SC_MODULE(Prod1) {
	sc_out<bool> out_signal;
	int p;

	SC_CTOR(Prod1) {
                p = 0;
		SC_THREAD(prod_behav);
	}

	void prod_behav() {
		std::cout << "Prod1: writing signal!\n";
		out_signal.write(true);
                p = 1;
	}
};

SC_MODULE(Prod2) {
	sc_out<bool> out_signal;
	int p;

	SC_CTOR(Prod2) {
                p = 0;
		SC_THREAD(prod_behav);
	}

	void prod_behav() {
		std::cout << "Prod2: writing signal!\n";
		out_signal.write(true);
                p = 1;
	}
};

SC_MODULE(Cons1) {
	sc_in<bool> in_signal;
	int c;

	SC_CTOR(Cons1) {
                c = 0;
		SC_THREAD(cons_behav);
	}

	void cons_behav() {
		wait(in_signal.value_changed_event());
		std::cout << "Cons1: signal received!\n";
                c = 1;
	}
};

SC_MODULE(Cons2) {
	sc_in<bool> in_signal1;
	sc_in<bool> in_signal2;
	int c;

	SC_CTOR(Cons2) {
                c = 0;
		SC_THREAD(cons_behav);
	}

	void cons_behav() {
		wait(in_signal1.value_changed_event() & in_signal2.value_changed_event());
		std::cout << "Cons2: signal1 AND signal2 received!\n";
                c = 1;
	}
};

SC_MODULE(Cons3) {
	sc_in<bool> in_signal1;
	sc_in<bool> in_signal2;
	int c;

	SC_CTOR(Cons3) {
                c = 0;
		SC_THREAD(cons_behav);
	}

	void cons_behav() {
		wait(in_signal1.value_changed_event() | in_signal2.value_changed_event());
		std::cout << "Cons3: signal1 OR signal2 received!\n";
                c = 1;
	}
};

SC_MODULE(top) {
	sc_signal<bool> signal1;
	sc_signal<bool> signal2;
	Cons1 cons1;
	Cons2 cons2;
	Cons3 cons3;
	Prod1 prod1;
	Prod2 prod2;

	SC_CTOR(top)
                : cons1("cons1"), cons2("cons2"), cons3("cons3"),
                  prod1("prod1"), prod2("prod2"),
                  signal1(false), signal2(false) {
		cons1.in_signal.bind(signal1);
		cons2.in_signal1.bind(signal1);
		cons2.in_signal2.bind(signal2);
		cons3.in_signal1.bind(signal1);
		cons3.in_signal2.bind(signal2);
		prod1.out_signal.bind(signal1);
		prod2.out_signal.bind(signal2);
	}
};

int sc_main(int argc, char** argv){
	top t("t");
	sc_start();
	assert(t.prod1.p == 1);
	assert(t.prod2.p == 1);
	assert(t.cons1.c == 1);
	assert(t.cons2.c == 1);
	assert(t.cons3.c == 1);
	return 0;
}

/* ex: set softtabstop=8 tabstop=8 shiftwidth=8 expandtab: */
