#include "systemc.h"

// +----------------------------------------+
// |                   TOP                  |
// |+-------------------++-------++--------+|
// ||        A1         ||  A2   ||   C    ||
// ||+-------++--------+||       ||        ||
// |||  M    ||   M    |||       ||        ||
// |||       ||        |||       ||        ||
// |||       ||        |||       ||        ||
// |||       ||        |||       ||        ||
// |||       ||        |||       ||        ||
// ||+-------++--------+||       ||        ||
// |+-------------------++-------++--------+|
// +----------------------------------------+

class InstanceEventConflictTableM: public sc_module {
public:

  SC_HAS_PROCESS(InstanceEventConflictTableM);
  sc_event &event_both_;
  sc_event &event_individual_;

  InstanceEventConflictTableM(
      sc_module_name mn,
      sc_event &event_both,
      sc_event &event_individual):
    sc_module(mn),
    event_both_(event_both),
    event_individual_(event_individual)
  {
    SC_THREAD(thread1);
  }

  void thread1()
  {
    wait(SC_ZERO_TIME);

    wait(event_both_);

    wait(event_individual_);
  }
};

class InstanceEventConflictTableA1: public sc_module {
public:

  SC_HAS_PROCESS(InstanceEventConflictTableA1);

  InstanceEventConflictTableM m1;
  InstanceEventConflictTableM m2;
  sc_event event_a_1;
  sc_event event_a_1_to_m1;
  sc_event event_a_1_to_m2;

  InstanceEventConflictTableA1(sc_module_name mn):
    sc_module(mn),
    m1("m1", event_a_1, event_a_1_to_m1),
    m2("m2", event_a_1, event_a_1_to_m2)
  {
    SC_THREAD(thread1);
  }

  void thread1()
  {
    event_a_1.notify(SC_ZERO_TIME);
    event_a_1_to_m1.notify(SC_ZERO_TIME);
    //event_a_1_to_m2.notify(SC_ZERO_TIME);
  }
};

class InstanceEventConflictTableA2: public sc_module {
public:

  SC_HAS_PROCESS(InstanceEventConflictTableA2);

  InstanceEventConflictTableA2(sc_module_name mn):
    sc_module(mn)
  {
    SC_THREAD(thread1);
  }

  void thread1()
  {
    wait(SC_ZERO_TIME);

    wait(SC_ZERO_TIME);
  }
};


class InstanceEventConflictTableC: public sc_module {
public:

  SC_HAS_PROCESS(InstanceEventConflictTableC);

  sc_event both;
  sc_event to_thread_4;
  sc_event to_thread_5;

  InstanceEventConflictTableC(sc_module_name mn):
    sc_module(mn)
  {
    //SC_THREAD(thread1);
    //SC_THREAD(thread2);
    SC_THREAD(thread3);
    SC_THREAD(thread4);
    SC_THREAD(thread5);
  }

  void thread1()
  {
    func();
  }

  void thread2()
  {
    func();
  }

  void func()
  {
    wait(SC_ZERO_TIME);

    wait(SC_ZERO_TIME);

    wait(SC_ZERO_TIME);

    wait();
  }

  void thread3()
  {
    both.notify(SC_ZERO_TIME);

    wait(SC_ZERO_TIME);

    to_thread_4.notify(SC_ZERO_TIME);
    to_thread_5.notify(SC_ZERO_TIME);

    wait(SC_ZERO_TIME);
    to_thread_4.notify(SC_ZERO_TIME);

    wait(SC_ZERO_TIME);
    to_thread_5.notify(SC_ZERO_TIME);
  }

  void thread4()
  {
    wait(both);

    wait(to_thread_4);

    wait(to_thread_4);
  }

  void thread5()
  {
    wait(both);

    wait(to_thread_5);

    wait(to_thread_5);
  }
};

class SomeClassA: public sc_module {

public:
  SC_HAS_PROCESS(SomeClassA);
  sc_event &event_a;

  SomeClassA(sc_module_name mn, sc_event &param_event_a):
      sc_module(mn),
      event_a(param_event_a) {

    SC_THREAD(thread1);
  }

  void thread1()
  {
    event_a.notify(SC_ZERO_TIME);
  }
};

class SomeClassB: public sc_module {

public:
  SC_HAS_PROCESS(SomeClassB);
  sc_event &event_b;

  SomeClassB(sc_module_name mn, sc_event &param_event_b):
      sc_module(mn),
      event_b(param_event_b) {

    SC_THREAD(thread1);
  }

  void thread1()
  {

    wait(event_b);
  }
};

class EventConflictTable: public sc_module {
public:

  SC_HAS_PROCESS(EventConflictTable);

  sc_event event_1;
  sc_event event_2;
  sc_event event_3;

  sc_event some_event;

  SomeClassA some_class_a;
  SomeClassB some_class_b;

  void thread1()
  {
    event_1.notify(SC_ZERO_TIME);

    wait(SC_ZERO_TIME);

    wait(event_2);

    wait(SC_ZERO_TIME);
  }

  void thread2()
  {
    wait(event_1);

    event_2.notify(SC_ZERO_TIME);
    wait(SC_ZERO_TIME);
    event_3.notify(SC_ZERO_TIME);
  }

  EventConflictTable(sc_module_name mn):
      sc_module(mn),
      some_class_a("some_class_a", some_event),
      some_class_b("some_class_b", some_event)
  {
    SC_THREAD(thread1);
    SC_THREAD(thread2);
  }
};

class SubModule: sc_module { // this class checks references in event lists

  SC_HAS_PROCESS(SubModule);

public:

  sc_event sub_event;
  sc_event &sub_event_ref;
  sc_event_and_list event_list;


  SubModule(sc_module_name mn, sc_event &event_ref):
    sc_module(mn),
    sub_event_ref(event_ref)
  {
    SC_THREAD(thread);
    SC_THREAD(thread1);
  }

  void before_end_of_elaboration()
  {
    event_list &= sub_event;
    event_list &= sub_event_ref;
  }

  void thread()
  {
    sub_event.notify(SC_ZERO_TIME);
  }

  void thread1()
  {
    wait(event_list);
  }
};

class EventLists: sc_module { // checks event lists without references

  SC_HAS_PROCESS(EventLists);

  SubModule sub_module_;
public:

  sc_event event_0;
  sc_event event_1;
  sc_event event_2;
  sc_event event_3;
  sc_event event_4;
  sc_event event_5;
  sc_event event_6;
  sc_event event_7;

  sc_event event_8;

  sc_event event_9;
  sc_event event_10;

  sc_event_and_list event_and_list;
  sc_event_or_list event_or_list;
  sc_event_or_list event_or_list_2;

  EventLists(sc_module_name mn):
    sc_module(mn),
    sub_module_("sub_module", event_8)
  {
    SC_THREAD(thread);
    SC_THREAD(thread1);
    SC_THREAD(thread2);

    SC_THREAD(thread3);
    SC_THREAD(thread4);
    SC_THREAD(thread5);
    SC_THREAD(thread6);
    SC_THREAD(thread7);
    SC_THREAD(thread8);
    SC_THREAD(thread9);
    SC_THREAD(thread10);
    SC_THREAD(thread11);
    SC_THREAD(thread12);
    SC_THREAD(thread13);
  }

  void before_end_of_elaboration()
  {
    event_and_list &= event_0;
    event_and_list &= event_1;
    event_and_list &= event_2;

    event_or_list |= event_3;
    event_or_list |= event_4;
    event_or_list |= event_5;

    event_or_list_2 |= event_9;
    event_or_list_2 |= event_10;
  }

  void thread()
  {
    wait(event_7);
  }
  void thread1()
  {
    wait(event_and_list);
  }
  void thread2()
  {
    wait(event_or_list);
  }


  void thread3()
  {
    event_0.notify(SC_ZERO_TIME);
  }
  void thread4()
  {
    event_1.notify(SC_ZERO_TIME);
  }
  void thread5()
  {
    event_2.notify(SC_ZERO_TIME);
  }
  void thread6()
  {
    event_3.notify(SC_ZERO_TIME);
  }
  void thread7()
  {
    event_4.notify(SC_ZERO_TIME);
  }
  void thread8()
  {
    event_5.notify(SC_ZERO_TIME);
  }
  void thread9()
  {
    event_6.notify(SC_ZERO_TIME);
  }
  void thread10()
  {
    event_7.notify(SC_ZERO_TIME);
  }

  void thread11()
  {
    event_8.notify(SC_ZERO_TIME);
  }

  void thread12()
  {
    event_9.notify(SC_ZERO_TIME);
    event_10.notify(SC_ZERO_TIME);
  }
  void thread13()
  {
    wait(event_or_list_2);
  }
};

class A: public sc_module {
public:
  SC_HAS_PROCESS(A);

  sc_event &refa_;

  A(sc_module_name mn, sc_event &refa):
    sc_module(mn),
    refa_(refa)
  {
    SC_THREAD(thread);
  }

  void thread()
  {
    refa_.notify(SC_ZERO_TIME);
  }
};

class B: public sc_module {
public:
  SC_HAS_PROCESS(B);

  sc_event &refb_;

  B(sc_module_name mn, sc_event &refb):
    sc_module(mn),
    refb_(refb)
  {
    SC_THREAD(thread);
  }

  void thread()
  {
    wait(refb_);
  }
};

class HostAB: public sc_module {
public:

  SC_HAS_PROCESS(HostAB);

  HostAB(sc_module_name mn, sc_event &ref_):
    sc_module(mn),
    a_("A", ref_),
    b_("B", ref_)
  {

  }

  A a_;
  B b_;
};

class Middle: public sc_module {

public:
  HostAB left_;
  HostAB right_;

  sc_event left_ref, right_ref;

  Middle(sc_module_name mn):
    sc_module(mn),
    left_("left", left_ref),
    right_("right", right_ref)
  {
  }
};

class MultipleInstancesTop: public sc_module {

public:
  Middle m1_;
  Middle m2_;

  MultipleInstancesTop(sc_module_name mn):
    sc_module(mn),
    m1_("m1"),
    m2_("m2")
  {
  }
};

class InstanceEventConflictTableTop: public sc_module {
public:

  InstanceEventConflictTableA1 a1;
  InstanceEventConflictTableA2 a2;
  InstanceEventConflictTableC c;
  EventConflictTable ect;
  EventLists el; // tests event lists event_1, ... event_10
  MultipleInstancesTop mit; // tests notification in A, B, HOSTAB style

  InstanceEventConflictTableTop(sc_module_name mn):
    sc_module(mn),
    a1("a1"),
    a2("a2"),
    c("c"),
    ect("ect"),
    el("el"),
    mit("mit")
  { }
};


#ifndef RISC_ALL_SEGMENT_TESTS
int sc_main(int argc, char **argv)
{
  InstanceEventConflictTableTop mm("mm");

  sc_start();
  return 0;
}
#endif

/* ex: set softtabstop=2 tabstop=2 shiftwidth=2 expandtab: */
